SystemOrganization addCategory: #'SqueakShell-Stdio'!
SystemOrganization addCategory: #'SqueakShell-Core'!
SystemOrganization addCategory: #'SqueakShell-Tools'!
SystemOrganization addCategory: #'SqueakShell-Support'!

Object subclass: #SqshCommandHistory
	instanceVariableNames: 'commands index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Support'!

!SqshCommandHistory methodsFor: 'editing' stamp: 'mt 5/7/2016 12:00'!
acceptCommand
	
	self updateCommand: self currentCommand.

	self commands add: ''.
	self index: self commands size.

	^ self commands last! !

!SqshCommandHistory methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:09'!
commands

	^ commands! !

!SqshCommandHistory methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:09'!
commands: anObject

	commands := anObject! !

!SqshCommandHistory methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:10'!
currentCommand

	^ self commands at: self index! !

!SqshCommandHistory methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:09'!
index

	^ index! !

!SqshCommandHistory methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:09'!
index: anObject

	index := anObject! !

!SqshCommandHistory methodsFor: 'initializing' stamp: 'mt 5/7/2016 11:12'!
initialize

	super initialize.

	self
		commands: (OrderedCollection with: String new);
		index: 1.! !

!SqshCommandHistory methodsFor: 'iterating' stamp: 'mt 5/7/2016 11:14'!
nextCommand

	self index: (self index + 1 min: self commands size).
	^ self currentCommand! !

!SqshCommandHistory methodsFor: 'iterating' stamp: 'mt 5/7/2016 11:14'!
previousCommand

	self index: (self index - 1 max: 1).
	^ self currentCommand! !

!SqshCommandHistory methodsFor: 'editing' stamp: 'mt 5/7/2016 11:16'!
updateCommand: anObject
	"Overwrite that current command only if we are not cycling through the history."
	
	(self index < self commands size and: [self commands last notEmpty])
		ifTrue: [self commands add: anObject]
		ifFalse: [self commands atLast: 1 put: anObject].

	"Ensure showing the current command."
	self index: self commands size.! !

Object subclass: #SqshControlManager
	instanceVariableNames: 'lastCycleTime interCycleDelay canvas scheduledControllers activeControllerStack keyboardInterpreter invalidRectangle exitValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Core'!

!SqshControlManager methodsFor: 'activate & deactivate' stamp: 'mt 7/8/2016 16:02'!
activate: aController
	"New active controller."
	
	self activeControllerStack push: aController.

	"Give controllers a chance to force a redraw"
	aController when: #display send: #handleScreenUpdate to: self.
	"Give views a chance to invalidate their contents."
	aController view when: #invalidate send: #invalidate: to: self.
	"Give controllers a chance to spawn other controllers."
	aController when: #spawn send: #activate: to: self.
	"Give controllers a chance to finish."
	aController when: #finish send: #deactivate: to: self with: aController.

	self invalidate; doLayout.
	aController activated.! !

!SqshControlManager methodsFor: 'activate & deactivate' stamp: 'mt 7/8/2016 16:15'!
activateControllerStack: anInteger
	"New active controller."
	
	self activeControllerStack: (self scheduledControllers at: anInteger ifAbsent: [1]).
	self ensureActiveController.
	self activeController activated.
	self invalidate; doLayout.! !

!SqshControlManager methodsFor: 'activate & deactivate' stamp: 'mt 7/8/2016 10:34'!
activeController

	self activeControllerStack ifNil: [^ nil].
	self activeControllerStack isEmpty ifTrue: [^ nil].
	^ self activeControllerStack top! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 7/8/2016 10:24'!
activeControllerStack

	^ activeControllerStack! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 7/8/2016 10:24'!
activeControllerStack: anObject

	activeControllerStack := anObject! !

!SqshControlManager methodsFor: 'geometry & layout' stamp: 'mt 7/7/2016 15:04'!
bounds

	^ canvas clipRect! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
canvas

	^ canvas! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
canvas: anObject

	canvas := anObject! !

!SqshControlManager methodsFor: 'activate & deactivate' stamp: 'mt 7/8/2016 10:40'!
deactivate: aController
	
	aController ~~ self activeController ifTrue: [^ self].
	
	aController
		removeActionsForEvent: #invalidate;
		removeActionsForEvent: #display;
		removeActionsForEvent: #finish;
		removeActionsForEvent: #spawn.
		
	self activeControllerStack pop.
	self ensureActiveController.
	self invalidate; doLayout.! !

!SqshControlManager methodsFor: 'display' stamp: 'mt 7/7/2016 14:37'!
display

	self displayOn: canvas.! !

!SqshControlManager methodsFor: 'display' stamp: 'mt 7/8/2016 09:49'!
displayOn: canvas

	invalidRectangle ifNil: [^ self].
	[canvas clipBy: invalidRectangle during: [:c | self activeController view displayOn: c]]
		ensure: [invalidRectangle := nil].! !

!SqshControlManager methodsFor: 'geometry & layout' stamp: 'mt 7/7/2016 15:15'!
doLayout

	self activeController ifNotNil: [:ac | ac doLayoutIn: self bounds].! !

!SqshControlManager methodsFor: 'main loop' stamp: 'mt 7/8/2016 09:51'!
doOneCycle
				
	self processDeferredMessages.
	self processInputEvents.
	self handleScreenUpdate.
	
	self tick.
	
	self interCyclePause.! !

!SqshControlManager methodsFor: 'activate & deactivate' stamp: 'mt 7/8/2016 10:55'!
ensureActiveController
	self activeController ifNil: [
		self activate: (SqshCommandLine newAtIndex: (self scheduledControllers indexOf: self activeControllerStack))].! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 7/8/2016 09:56'!
exitValue
	^ exitValue! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 7/8/2016 09:56'!
exitValue: anObject
	exitValue := anObject.! !

!SqshControlManager methodsFor: 'events' stamp: 'mt 7/8/2016 10:54'!
handleKeyboardEvent: event

	| shiftPressed controlKeyPressed commandKeyPressed keyCharacter |
	shiftPressed := (event fifth bitShift: 3) anyMask: 8.
	controlKeyPressed := (event fifth bitShift: 3) anyMask: 16.
	commandKeyPressed := (event fifth bitShift: 3) anyMask: 64.
	keyCharacter := event third asCharacter.
	
	"Quit the Squeak Shell."
	keyCharacter = Character escape
		ifTrue: [
			Project current isSqueakShell
				ifTrue: [^ Project returnToParentProject]
				ifFalse: [^ self exitValue: #escaped]].

	(controlKeyPressed and: [keyCharacter = Character arrowUp])
		ifTrue: [^ self activeController handleMouseWheel: #up].
	(controlKeyPressed and: [keyCharacter = Character arrowDown])
		ifTrue: [^ self activeController handleMouseWheel: #down].		
	(controlKeyPressed and: [keyCharacter asciiValue between: 49 and: 57])
		ifTrue: [^ self activateControllerStack: keyCharacter asciiValue - 48].

	event fourth = 0 "key stroke"
		ifTrue: [self activeController
			handleKeyboard: (keyboardInterpreter nextCharFrom: EventSensor default firstEvt: event)
			modifiers: {shiftPressed. controlKeyPressed. commandKeyPressed}].! !

!SqshControlManager methodsFor: 'events' stamp: 'mt 7/7/2016 14:28'!
handleMouseEvent: event

	"No mouse support."! !

!SqshControlManager methodsFor: 'main loop' stamp: 'mt 7/8/2016 09:52'!
handleScreenUpdate

	invalidRectangle ifNil: [^ self].

	Display deferUpdates: true.

	self displayOn: canvas.
		
	canvas finish. "Really needed?"
	
	Display
		deferUpdates: false;
		forceToScreen.! !

!SqshControlManager methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 10:34'!
initialize

	super initialize.

	keyboardInterpreter := LanguageEnvironment currentPlatform class defaultInputInterpreter.
	scheduledControllers := (1 to: 9) collect: [:ea | Stack new].
	
	self reset.
	self activateControllerStack: 1.! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
interCycleDelay

	^ interCycleDelay! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
interCycleDelay: anObject

	interCycleDelay := anObject! !

!SqshControlManager methodsFor: 'main loop' stamp: 'mt 7/7/2016 14:26'!
interCyclePause

	| millisecondsToWait minimumSpan |	
	minimumSpan := (Preferences higherPerformance ifTrue: [0] ifFalse: [20]).
	
	millisecondsToWait := lastCycleTime
		ifNil: [0] ifNotNil: [:t | minimumSpan - (Time millisecondsSince: t) abs].
		
	(millisecondsToWait > 0 and: [millisecondsToWait <= minimumSpan]) ifTrue: [
		(interCycleDelay isNil or: [ interCycleDelay beingWaitedOn ])
			ifTrue: [ interCycleDelay := Delay forMilliseconds: millisecondsToWait ]
			ifFalse: [ interCycleDelay delayDuration: millisecondsToWait ].
		interCycleDelay wait ].
	
	lastCycleTime := Time millisecondClockValue.! !

!SqshControlManager methodsFor: 'display' stamp: 'mt 7/8/2016 10:39'!
invalidate

	invalidRectangle := self bounds.! !

!SqshControlManager methodsFor: 'display' stamp: 'mt 7/8/2016 09:43'!
invalidate: rectangle
	"Redraw the given area."

	invalidRectangle := invalidRectangle
		ifNil: [rectangle]
		ifNotNil: [:rect | rect quickMerge: rectangle].! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
keyboardInterpreter

	^ keyboardInterpreter! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
keyboardInterpreter: anObject

	keyboardInterpreter := anObject! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleTime

	^ lastCycleTime! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleTime: anObject

	lastCycleTime := anObject! !

!SqshControlManager methodsFor: 'main loop' stamp: 'mt 7/7/2016 14:26'!
processDeferredMessages

	| object |
	[object := self activeController deferredMessages nextOrNil]
		whileNotNil: [object value].! !

!SqshControlManager methodsFor: 'main loop' stamp: 'mt 2/15/2016 17:57'!
processInputEvents

	| event |
	[event := EventSensor default nextEvent] whileNotNil: [

		event first = 1 "mouse event"
			ifTrue: [self handleMouseEvent: event].
	
		event first = 2 "keyboard event"
			ifTrue: [self handleKeyboardEvent: event].
	
	].! !

!SqshControlManager methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 10:41'!
reset

	canvas := Display getCanvas.
	self invalidate; doLayout.! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 20:37'!
scheduledControllers

	^ scheduledControllers! !

!SqshControlManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 18:32'!
scheduledControllers: anObject

	scheduledControllers := anObject! !

!SqshControlManager methodsFor: 'main loop' stamp: 'mt 7/6/2016 15:10'!
tick

	self activeController tick: Time millisecondClockValue.! !

SqshControlManager subclass: #SqshStdioManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Stdio'!

!SqshStdioManager methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 17:48'!
activate: aController
	"Remove subviews."
	
	aController view subviews: #(). "Looks awkward."
	super activate: aController.! !

!SqshStdioManager methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 17:55'!
processInputEvents

	| char |
	[
	[FileStream stdin peek ifNil: [^ self]] on: Error do: [^ self].
	char := FileStream stdin next
	
	] whileNotNil: [
		self handleKeyboardEvent: {2. 0. char asciiValue. 0 "keystroke". 2r0000. char asciiValue}.
		^ self ].! !

!SqshStdioManager methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 17:32'!
reset

	canvas := SqshStdioCanvas on: (Form extent: 300@200 depth: 32).
	self invalidate; doLayout.! !

Object subclass: #SqshController
	instanceVariableNames: 'model view deferredMessages index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Core'!

SqshController subclass: #SqshCommandLine
	instanceVariableNames: 'history receivers isReplacingPrompt topLeftStatusView topLeftStatus topRightStatusView topRightStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Tools'!

!SqshCommandLine methodsFor: 'evaluation' stamp: 'mt 7/6/2016 17:15'!
acceptCommand

	| command |
	command := self history currentCommand.
	isReplacingPrompt := true.
	
	[command ifNotEmpty: [:cmd | self evaluate: cmd]]
		ensure: [
			command ifNotEmpty: [self history acceptCommand].
			self prompt: self history currentCommand.
			
			"Receiver might have changed during evaluation..."
			self updatePrompt].! !

!SqshCommandLine methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 16:02'!
activated
	self updateStatus.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:50'!
clearScreen

	<shortcuts: #(clear cls) description: 'Clear screen and output history'>
	
	self model clearLines.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
debugIt: expression

	<shortcuts: #('d') description: 'Debug {1}'>

	| methodNode |
	methodNode := [Compiler new
		compileNoPattern: expression
		in: self model receiver class
		context: self model context
		notifying: self
		ifFail: [^nil]]
			on: OutOfScopeNotification
			do: [:ex | ex resume: true].

	ToolSet
		debugMethod: methodNode generateWithTempNames
		forReceiver: self model receiver
		inContext: self model context.! !

!SqshCommandLine methodsFor: 'geometry & layout' stamp: 'mt 7/8/2016 16:18'!
doLayoutIn: newBounds
	"Custom layout strategy. Position the two status views at the top and bottom edges."
	
	super doLayoutIn: newBounds.

	topLeftStatusView bounds: (newBounds topLeft extent: newBounds width @ topLeftStatusView defaultTextFont height).
	topRightStatusView bounds: (newBounds topLeft extent: newBounds width @ topRightStatusView defaultTextFont height).! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
editMethodOrClass: codeToMethodOrClass

	<shortcuts: #('ed') description: 'Edit the method or class {1}'>
		
	| thing method class selector category |
	thing := self evaluate: codeToMethodOrClass.

	thing isBehavior ifTrue: [
		class := thing.
		(self spawn: SqshTextEditor) in: [:textEditor |
			textEditor
				contents: class definition asString;
				saveBlock: [:text | self evaluate: text asString]].
		^ self].

	thing isCompiledMethod ifFalse: [
		"Method references..."
		thing := thing compiledMethod].
	
	method := thing.
	class := method methodClass.
	selector := method selector.
	category := class organization categoryOfElement: selector.

	(self spawn: SqshTextEditor) in: [:textEditor |
		textEditor
			contents: method getSource;
			saveBlock: [:text |  class compile: text classified: category]].! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
editText

	<shortcuts: #('ed') description: 'Edit some text'>
	
	(self spawn: SqshTextEditor) in: [:textEditor |
		textEditor
			contents: (self model bindings at: #'_textEditor' ifAbsent: ['']);
			saveBlock: [:text | self model bindings at: #'_textEditor' put: text]]! !

!SqshCommandLine methodsFor: 'evaluation' stamp: 'mt 7/8/2016 14:51'!
evaluate: string
	"Override to support shortcuts and re-binding of 'self'."

	| code newReceiver result |
	string ifEmpty: [^ self].
	
	code := string.
	newReceiver := false.

	(code beginsWith: '!!') ifTrue: [
		(self processShortcut: code allButFirst)
			ifTrue: [^ self].

		code = '!!!!' ifTrue: [^ self model receiver: (self receivers ifEmpty: [nil] ifNotEmpty: [:r | r removeLast])].
	
		newReceiver := true.
		code := code allButFirst].

	result := super evaluate: code.

	newReceiver
		ifFalse: [self showMessage: result printString]
		ifTrue: [
			self receivers add: result.
			self model receiver: result].
	^ result! !

!SqshCommandLine methodsFor: 'events' stamp: 'mt 7/6/2016 15:45'!
handleKeyboard: keyCharacter modifiers: someModifiers

	"Evaluate the input."
	(keyCharacter = Character enter or: [keyCharacter = Character cr])
		ifTrue: [
			someModifiers first "shift"
				ifFalse: [^ self acceptCommand]].

	"Command history."
	keyCharacter = Character arrowUp
		ifTrue: [
			self prompt contents: self history previousCommand.
			self model moveCursorEnd.
			^ self].
	keyCharacter = Character arrowDown
		ifTrue: [
			self prompt contents: self history nextCommand.
			self model moveCursorEnd.
			^ self].
	
	super handleKeyboard: keyCharacter modifiers: someModifiers.
	self history updateCommand: self prompt contents asString.! !

!SqshCommandLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:53'!
history

	^ history! !

!SqshCommandLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:53'!
history: anObject

	history := anObject! !

!SqshCommandLine methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 16:19'!
initialize

	super initialize.
	
	self initializeStatus.
	self view margin: (0 @ topLeftStatusView defaultTextFont height corner: 0 @ 0).
	
	self
		history: SqshCommandHistory new;
		receivers: OrderedCollection new;
		showWelcomeMessage;
		updatePrompt;
		updateStatus.! !

!SqshCommandLine methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 16:08'!
initializeStatus

	| topLeftStatusModel topRightStatusModel |
	
	topLeftStatusModel :=SqshModel new.
	topLeftStatus := topLeftStatusModel addLine: ''.
	topLeftStatusView := SqshView new
		model: topLeftStatusModel;
		fillStyle: (Color gray: 0.9);
		defaultTextFont: (StrikeFont familyName: 'Bitmap DejaVu Sans' pointSize: 9);
		defaultTextColor: (Color gray: 0.4);
		showCursor: false;
		autoScroll: false;
		yourself.
		
	topRightStatusModel :=SqshModel new.
	topRightStatus := topRightStatusModel addLine: ''.
	topRightStatusView := SqshView new
		model: topRightStatusModel;
		fillStyle: Color transparent;
		defaultTextFont: (StrikeFont familyName: 'Bitmap DejaVu Sans' pointSize: 9);
		defaultTextColor: (Color gray: 0.4);
		showCursor: false;
		autoScroll: false;
		yourself.
		
	self view subviews: {topLeftStatusView. topRightStatusView}! !

!SqshCommandLine methodsFor: 'evaluation' stamp: 'mt 7/8/2016 15:39'!
processShortcut: code
	
	| shortcut argIndex |
	argIndex := code indexOf: Character space.
	shortcut := argIndex > 1
		ifTrue: [code copyFrom: 1 to: argIndex - 1]
		ifFalse: [code].

	self shortcuts do: [:pragma | pragma arguments first in: [:pattern |
		(pattern includes: shortcut)
			ifTrue: [
				(pragma selector numArgs = 0 and: [argIndex <= 1])
					ifTrue: [
						self executeMethod: pragma method.
						^ true].
				(pragma selector numArgs = 1 and: [argIndex > 1])
					ifTrue: [
						self
							with: (code allButFirst: shortcut size + 1) withBlanksTrimmed
							executeMethod: pragma method.
						^ true]]]].
	^ false! !

!SqshCommandLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:17'!
prompt

	^ self model currentLine! !

!SqshCommandLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 13:36'!
prompt: string
	"Set a new prompt. Use the same prefix."

	isReplacingPrompt := true.
	
	[self model currentLine: ((self model addLine: string)
								prefix: self model currentLine prefix;
								yourself).
	] ensure: [isReplacingPrompt := false].! !

!SqshCommandLine methodsFor: 'printing' stamp: 'mt 5/7/2016 12:27'!
receiverString
	
	^ ((self receivers ifEmpty: [{nil}]) collect: [:object | '[', (object printStringLimitedTo: 10), ']'])
		join, '>'! !

!SqshCommandLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 12:24'!
receivers

	^ receivers! !

!SqshCommandLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 12:24'!
receivers: anObject

	receivers := anObject! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 5/7/2016 17:49'!
revertLastChange

	<shortcuts: #(undo revert) description: 'Revert the last method change'>
	
	RecentMessages default revertMostRecent.
	self inform: 'reverted: ', RecentMessages default mostRecent.! !

!SqshCommandLine methodsFor: 'evaluation' stamp: 'mt 5/7/2016 18:08'!
shortcuts
	"Shortcuts are methods that provide a special pragma mentioning the shortcut and a description."
	
	^ Array streamContents: [:strm |
		self class methodsDo: [:method |
			method pragmas do: [:pragma |
				(pragma keyword = #shortcut:description: or: [pragma keyword = #shortcuts:description:])
					ifTrue: [strm nextPut: pragma]]]]! !

!SqshCommandLine methodsFor: 'printing' stamp: 'mt 5/7/2016 18:18'!
shortcutsString

	| shortcuts max placeholder |
	shortcuts := self shortcuts.
	max := 0.
	placeholder := '<arg>'. "Should be the message argument, though."
	"
	shortcuts do: [:pragma | (pragma argumentAt: 1) in: [:pattern |
		pattern isArray
			ifTrue: [pattern do.."! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
showBindings

	<shortcuts: #('bindings') description: 'List all bindings'>

	| max |
	max := 4. "self"
	self model bindings keys do: [:key | max := max max: key size].
	max := max + 3.
	
	(self showMessage: ('self' padded: #right to: max with: $.), (self model receiver printStringLimitedTo: 40))
		setMonospaced.
	
	self model bindings keysAndValuesDo: [:identifier :object |
		(self showMessage: (identifier padded: #right to: max with: $.), (object printStringLimitedTo: 40))
			setMonospaced].! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:42'!
showHelp

	<shortcuts: #('help') description: 'Show this help'>

	| len sortedShortcuts |
	sortedShortcuts := self shortcuts sortBy: [:s1 :s2 | (s1 argumentAt: 1) first <= (s2 argumentAt: 1) first].
	len := 3 + ((sortedShortcuts gather: [:ea | ea argumentAt: 1]) collect: [:ea | ea size]) max.
	
	(self showMessage: ('\The Squeak Shell - Command Line\*******************************\\GENERAL\                Esc ... Leave this project and enter the parent project if any\            !!<code> ... Bind ''self'' to the result of the <code>\                 !!!! ... Use previous binding for ''self''\    <var> := <code> ... Bind <var> to the result of <code>\            _result ... Result of last expression\           Arrow-Up ... Previous command\         Arrow-Down ... Next command\           Wheel-Up ... Scroll up and deactivate auto-scroll\         Wheel-Down ... Scroll down and activate auto-scroll\       Return/Enter ... Evaluate the command\ Shift+Return/Enter ... Insert line break\              CMD+V ... Paste from the clipboard\\SHORTCUTS\{1}\ ' withCRs format: {
	
	(sortedShortcuts gather: [:ea |
		(ea argumentAt: 1) collect: [:id |
			('!!', id padded: #left to: len with: Character space), ' ... ', (ea argumentAt: 2)]])
				joinSeparatedBy: String cr	
	
	})) setMonospaced.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
showImplementorsOf: symbol

	<shortcuts: #('m') description: 'Implementors of {1}'>

	self model lastResult: (SystemNavigation default allImplementorsOf: symbol asSymbol).
	(self showMessage: 'Implementors of ''', symbol, '''') setMonospaced.
	(self showMessage: self model lastResult printNumberedLines) setMonospaced.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
showLogo

	<shortcuts: #('logo') description: 'Guess what?!!'>

	(self showMessage: SqueakShell logo) setMonospaced.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:38'!
showMessagesWith: pattern

	<shortcuts: #('sel') description: 'Find messages with the pattern {1}'>

	ToolSet browseMessageNames: pattern.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:39'!
showSendersOf: symbol

	<shortcuts: #('n') description: 'Senders of {1}'>

	self model lastResult: (SystemNavigation default allCallsOn: symbol asSymbol). 
	(self showMessage: 'Senders of ''', symbol, '''') setMonospaced.
	(self showMessage: self model lastResult printNumberedLines) setMonospaced.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:39'!
showWelcomeMessage

	<shortcuts: #('welcome') description: 'The welcome message.'>
	
	self showLogo.
		
	self showMessage: ('Welcome to the Squeak Shell #{1}!!\\Switch between shells with [Ctrl]+[1-9].\Type !!help to get some help.\If you are brave enough, try evaluating some Smalltalk code.\You can leave anytime by hitting [Escape].\\' withCRs format: {self index}).! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:39'!
snapshot
    "This is a comment."
    <shortcuts: #('s') description: 'Save the image'>
    
    Smalltalk snapshot: true andQuit: false.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:39'!
snapshotAndQuit

	<shortcuts: #('q') description: 'Save the image and quit'>
	
	Smalltalk snapshot: true andQuit: true.! !

!SqshCommandLine methodsFor: 'updating' stamp: 'mt 7/8/2016 16:03'!
tick: msecs

	| time |
	time := String streamContents: [:s |
		Time now print24: true showSeconds: false on: s].
	time = topRightStatus contents string "For performance!!"
		ifFalse: [topRightStatus contents: time; setRightFlush].! !

!SqshCommandLine methodsFor: 'updating' stamp: 'mt 5/7/2016 13:37'!
update: aspect

	aspect == #lines ifTrue: [
		((isReplacingPrompt ~~ true and: [self prompt notNil]) and: [self model lines last ~~ self prompt]) ifTrue: [
			| tmp |
			"Ensure prompt always at last."
			tmp := self model lines remove: self prompt.
			self model lines addLast: tmp.
			self model cursor: (self model cursor x @ self model lines size)]].
		
	super update: aspect.! !

!SqshCommandLine methodsFor: 'shortcuts' stamp: 'mt 7/8/2016 15:39'!
updateFromServer

	<shortcuts: #(up update) description: 'Update the image from the Internet'>
	
	MCMcmUpdater updateFromServer.! !

!SqshCommandLine methodsFor: 'updating' stamp: 'mt 7/6/2016 15:26'!
updatePrompt
	"Sync prompt prefix with current receiver."
	
	self prompt prefix: self receiverString.	! !

!SqshCommandLine methodsFor: 'updating' stamp: 'mt 7/8/2016 16:25'!
updateStatus

	"Current index."
	| controllerStacks |
	self flag: #refactor. "mt: No access to global state..."
	controllerStacks := Project current isSqueakShell
		ifTrue: [Project current world scheduledControllers]
		ifFalse: [(Array new: 9 withAll: #()) at: 1 put: (Stack new push: self; yourself); yourself].
	
	topLeftStatus contents: (String streamContents: [:s |
		controllerStacks doWithIndex: [:ea :index |
			s nextPutAll: ('{1} {2} {3}' format: {
				ea isEmpty ifTrue: [' '] ifFalse: [ea top == self ifTrue: ['['] ifFalse: [' ']].
				ea isEmpty ifTrue: ['*'] ifFalse: [index].
				ea isEmpty ifTrue: [' '] ifFalse: [ea top == self ifTrue: [']'] ifFalse: [' ']].}).
			s nextPut: Character space]]).

	topLeftStatus setMonospaced.! !

!SqshController class methodsFor: 'instance creation' stamp: 'mt 7/8/2016 10:54'!
newAtIndex: anInteger
	^ self basicNew
		index: anInteger;
		initialize;
		yourself! !

!SqshController methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 16:02'!
activated
	"Use this call-back to react when being activated."! !

!SqshController methodsFor: 'evaluation' stamp: 'mt 2/14/2016 13:42'!
bindingOf: aString

	^ self model bindingOf: aString! !

!SqshController methodsFor: 'initialize-release' stamp: 'mt 7/6/2016 15:33'!
captureDisplay
	"Show the current contents of the display in the background."
	
	self view
		fillStyle: (BitmapFillStyle fromForm: (Display copy getCanvas in: [:canvas |
			canvas fillRectangle: (0@0 extent: canvas form extent) color: (Color white alpha: 0.9).
			canvas form]))! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:56'!
deferredMessages

	^ deferredMessages! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:56'!
deferredMessages: anObject

	deferredMessages := anObject! !

!SqshController methodsFor: 'geometry & layout' stamp: 'mt 7/8/2016 15:22'!
doLayoutIn: newBounds
	"Default layout strategy. Just use the whole bounds."
	
	self view bounds: newBounds.! !

!SqshController methodsFor: 'evaluation' stamp: 'mt 5/7/2016 13:11'!
evaluate: string

	| result |
	string ifEmpty: [^ self].
	
	result := Compiler new
		evaluate: string
		in: self model context
		to: self model receiver
		notifying: self
		ifFail: [^ self]
		logged: true.

	self model lastResult: result.
	^ result! !

!SqshController methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 11:03'!
finish

	self triggerEvent: #finish.! !

!SqshController methodsFor: 'events' stamp: 'mt 7/8/2016 15:00'!
handleKeyboard: keyCharacter modifiers: someModifiers
	
	"Text correction."
	keyCharacter = Character backspace
		ifTrue: [^ self model backspace; moveCursorLeft].
	keyCharacter = Character delete
		ifTrue: [^ self model forwardDelete].
	(keyCharacter = $v and: [someModifiers third "CMD" | someModifiers second "CTRL"])
		ifTrue: [^ self model insert: Clipboard clipboardText asString].
	
	"Text cursor movement."
	keyCharacter = Character arrowLeft
		ifTrue: [^ self model moveCursorLeft].
	keyCharacter = Character arrowRight
		ifTrue: [^ self model moveCursorRight].
	keyCharacter = Character arrowUp
		ifTrue: [^ self model moveCursorUp].
	keyCharacter = Character arrowDown
		ifTrue: [^ self model moveCursorDown].
	keyCharacter = Character home
		ifTrue: [^ self model moveCursorPos1].
	keyCharacter = Character end
		ifTrue: [^ self model moveCursorEnd].
		
	"Whatever you type, show the things around the text cursor."
	self view autoScroll: true.

	"Simulate tabs."
	keyCharacter = Character tab
		ifTrue: [self model insert: '    '. 4 timesRepeat: [self model moveCursorRight]. ^ self].
	
	"Regular character input."
	self model insert: keyCharacter asString.
	self model moveCursorRight.! !

!SqshController methodsFor: 'events' stamp: 'mt 2/14/2016 16:55'!
handleMouseWheel: upOrDown

	upOrDown = #up
		ifTrue: [self view scrollUp: 5 "lines"].
	upOrDown = #down
		ifTrue: [self view scrollDown: 5 "lines"].! !

!SqshController methodsFor: 'accessing' stamp: 'mt 7/8/2016 10:35'!
index
	^ index! !

!SqshController methodsFor: 'accessing' stamp: 'mt 7/8/2016 10:35'!
index: anInteger
	index := anInteger.! !

!SqshController methodsFor: 'initialize-release' stamp: 'mt 7/6/2016 13:08'!
initialize

	super initialize.

	self
		deferredMessages: SharedQueue new;
		model: SqshModel new;
		view: SqshView new.

	self model currentLine: (self model addLine: '').
			
	self view model: self model.! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
model

	^ model! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:10'!
model: anObject

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].

	model := anObject! !

!SqshController methodsFor: 'evaluation' stamp: 'mt 2/17/2016 17:15'!
notify: msg at: pos in: stream

	| source |
	source := stream contents.
	
	self showMessage: (source copyFrom: 1 to: pos-1), String cr, msg, String cr, (source copyFrom: pos to: source size).! !

!SqshController methodsFor: 'printing' stamp: 'mt 7/6/2016 15:18'!
showMessage: stringOrText
	
	^ self model addLine: stringOrText! !

!SqshController methodsFor: 'printing' stamp: 'mt 7/8/2016 09:37'!
showMessageDuring: workBlock

	^ workBlock value: [:info |
		self showMessage: info asStringOrText.
		self triggerEvent: #display]! !

!SqshController methodsFor: 'printing' stamp: 'mt 7/8/2016 09:37'!
showProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock

	| result progress lineLabel lineProgress |
	lineLabel := self model addLine: (titleString ifEmpty: ['Please wait...']).
	lineProgress := self model addLine: ''.
	
	progress := [:barValue | | p |
		p := (barValue min: maxVal) - minVal / (maxVal - minVal max: 1.0).
		lineProgress contents: ('{1}% [{2}>{3}] {4}/{5}' format: {
			(p * 100) rounded asString padded: #left to: 3 with: Character space.
			String new: (p * 40) rounded withAll: $=.
			String new: (1-p * 40) rounded withAll: Character space.
			barValue.
			maxVal.}).
		lineProgress setMonospaced.
		self triggerEvent: #display].

	[result := workBlock value: progress]
		on: ProgressNotification
		do: [:ex |
			ex extraParam isString ifTrue: [lineLabel contents: ex extraParam].
			ex resume].

	progress value: maxVal.

	^ result! !

!SqshController methodsFor: 'printing' stamp: 'mt 7/6/2016 15:21'!
showWarning: stringOrText

	(self showMessage: stringOrText)
		setColor: Color red;
		setMonospaced.! !

!SqshController methodsFor: 'initialize-release' stamp: 'mt 7/11/2016 10:20'!
spawn: aControllerClass

	| newController |
	newController := aControllerClass newAtIndex: self index.
	self triggerEvent: #spawn with: newController.
	^ newController! !

!SqshController methodsFor: 'updating' stamp: 'mt 7/6/2016 15:10'!
tick: milliseconds
	"Perform some frequent updates."! !

!SqshController methodsFor: 'updating' stamp: 'mt 7/8/2016 09:37'!
update: aspect

	super update: aspect.
	
	(aspect = #appendEntry or: [aspect = #appendEntryLater])
		ifTrue: [
			self showMessage: Transcript contents.

			"Force screen update."
			(aspect = #appendEntry or: [Processor activeProcess == Project current uiProcess])
				ifTrue: [self triggerEvent: #display]].! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
view

	^ view! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
view: anObject

	view := anObject! !

!SqshController methodsFor: 'evaluation' stamp: 'mt 2/17/2016 16:49'!
wantsInteractiveErrorCorrection

	^ false! !

SqshController subclass: #SqshTextEditor
	instanceVariableNames: 'topStatusView topStatus bottomStatusView bottomStatus saveBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Tools'!

!SqshTextEditor methodsFor: 'accessing' stamp: 'mt 7/6/2016 14:12'!
bottomStatus
	^ bottomStatus! !

!SqshTextEditor methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:11'!
contents
	^ self model asString! !

!SqshTextEditor methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:55'!
contents: aString
	self model replaceLines: (aString asString copyReplaceAll: String tab with: '    ') lines.! !

!SqshTextEditor methodsFor: 'geometry & layout' stamp: 'mt 7/7/2016 15:06'!
doLayoutIn: newBounds
	"Custom layout strategy. Position the two status views at the top and bottom edges."
	
	super doLayoutIn: newBounds.

	topStatusView bounds: (newBounds topLeft extent: newBounds width @ topStatusView defaultTextFont height).
	bottomStatusView bounds: (newBounds bottomLeft - (0@bottomStatusView defaultTextFont height) corner: newBounds bottomRight).! !

!SqshTextEditor methodsFor: 'events' stamp: 'mt 7/8/2016 14:16'!
handleKeyboard: keyCharacter modifiers: someModifiers

	"Evaluate the current line as code."
	((keyCharacter = Character enter or: [keyCharacter = Character cr])
		and: [someModifiers first "shift"])
			ifTrue: [
				self evaluate: self model currentLine contents string.
				self updateStatus.
				^ self].

	"Insert/add lines never add line breaks to the line in the model itself."
	(keyCharacter = Character enter or: [keyCharacter = Character cr])
		ifTrue: [^ self model split].
	
	"Remove lines."
	(keyCharacter = Character delete)
		ifTrue: [self model joinForeward ifTrue: [^ self]].
	(keyCharacter = Character backspace)
		ifTrue: [self model joinBackward ifTrue: [^ self]].


	"Quit the editor."
	(keyCharacter = $q and: [someModifiers third "cmd/alt"])
		ifTrue: [^ self finish].
	
	"Save the changes."
	(keyCharacter = $s and: [someModifiers third "cmd/alt"])
		ifTrue: [^ self save].	

	"Additional cursor movement."
	keyCharacter = Character arrowUp
		ifTrue: [^ self model moveCursorUp].
	keyCharacter = Character arrowDown
		ifTrue: [^ self model moveCursorDown].
	(keyCharacter = Character arrowLeft and: [self model cursor x = 0 and: [self model cursor y > 1]])
		ifTrue: [^ self model moveCursorUp; moveCursorEnd].
	(keyCharacter = Character arrowRight and: [self model cursorAtEndOfLine and: [self model cursorAtLastLine not]])
		ifTrue: [^ self model moveCursorDown; moveCursorPos1].
	
	super handleKeyboard: keyCharacter modifiers: someModifiers.! !

!SqshTextEditor methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 16:05'!
initialize

	super initialize.

	self initializeStatus.
		
	self saveBlock: [:text | "ignore"].
	
	self view
		margin: (0 @ topStatusView defaultTextFont height corner: 0 @ bottomStatusView defaultTextFont height);
		defaultTextFillStyle: (SolidFillStyle color: (Color gray: 0.95)).
		
	self updateStatus.! !

!SqshTextEditor methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 16:32'!
initializeStatus

	| topStatusModel bottomStatusModel |
	
	topStatusModel :=SqshModel new.
	topStatus := topStatusModel addLine: ''.
	topStatusView := SqshView new
		model: topStatusModel;
		fillStyle: (Color gray: 0.3);
		defaultTextFont: Preferences standardDefaultTextFont;
		defaultTextColor: (Color gray: 0.9);
		showCursor: false;
		autoScroll: false;
		yourself.
		
	bottomStatusModel :=SqshModel new.
	bottomStatus := bottomStatusModel addLine: ''.
	bottomStatusView := SqshView new
		model: bottomStatusModel;
		fillStyle: (Color gray: 0.3);
		defaultTextFont: Preferences standardDefaultTextFont;
		defaultTextColor: (Color gray: 0.9);
		showCursor: false;
		autoScroll: false;
		yourself.
		
	self view subviews: {topStatusView. bottomStatusView}! !

!SqshTextEditor methodsFor: 'events' stamp: 'mt 7/8/2016 14:16'!
save
	self saveBlock value: self model asString.
	self model clearEditFlag.
	self updateStatus.! !

!SqshTextEditor methodsFor: 'accessing' stamp: 'mt 7/8/2016 11:14'!
saveBlock
	^ saveBlock! !

!SqshTextEditor methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:05'!
saveBlock: aBlock
	saveBlock := aBlock.! !

!SqshTextEditor methodsFor: 'as yet unclassified' stamp: 'mt 7/6/2016 15:52'!
showClockAndDate
	"Show the current date and time as an overlay buffer."
	
	self clock: (self model addLine: '').
	self clock
		alignment: #right;
		font: (StrikeFont familyName: 'BitstreamVeraSansMono' pointSize: 12).

	self date: (self model addLine: '').
	self date
		contents: Date current weekday, ', ', Date current printString;
		alignment: #right;
		color: Color gray;
		font: (StrikeFont familyName: 'BitstreamVeraSansMono' pointSize: 12).! !

!SqshTextEditor methodsFor: 'accessing' stamp: 'mt 7/6/2016 14:12'!
topStatus
	^ topStatus! !

!SqshTextEditor methodsFor: 'updating' stamp: 'mt 7/8/2016 12:31'!
update: aspect

	(#(cursor lines) includes: aspect)
		ifTrue: [self updateStatus].! !

!SqshTextEditor methodsFor: 'updating' stamp: 'mt 7/8/2016 14:29'!
updateBottomStatus

	bottomStatus ifNil: [^ self].
	bottomStatus contents: ('{1} @ {2} [{3}]      Hit [Shift]+[Enter] to evaluate: {4}' format: {
		self model cursor x asString padded: #left to: 2 with: Character space.
		self model cursor y asString padded: #left to: 2 with: Character space.
		self model lines size asString padded: #left to: 3 with: Character space.
		self model lastResult printString withoutLineEndings truncateWithElipsisTo: 20}).
	bottomStatus setMonospaced.! !

!SqshTextEditor methodsFor: 'updating' stamp: 'mt 7/8/2016 11:52'!
updateStatus

	self updateTopStatus; updateBottomStatus.! !

!SqshTextEditor methodsFor: 'updating' stamp: 'mt 7/8/2016 16:33'!
updateTopStatus

	topStatus ifNil: [^ self].
	topStatus contents: ('The Squeak Shell - Text Editor     {1}Hit [CMD]+[S] to save, [CMD]+[Q] to quit' format: {
		self model hasEdits ifFalse: [''] ifTrue: ['*unsaved edits*       ']}).! !

Object subclass: #SqshModel
	instanceVariableNames: 'lines receiver context cursor bindings currentLine edits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Core'!

!SqshModel methodsFor: 'adding' stamp: 'mt 7/6/2016 16:04'!
addLine: stringOrText

	^ (self addLines: {stringOrText}) first! !

!SqshModel methodsFor: 'adding' stamp: 'mt 7/6/2016 15:52'!
addLines: someLines

	| result |
	result := self lines addAll: (someLines collect: [:line |
		SqshLine new
			visible: true;
			contents: line;
			prefix: '';
			addDependent: self;
			yourself]).
	self changed: #lines.
	^ result! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:04'!
asString
	^ (self lines collect: [:ea | ea contents asString])
		joinSeparatedBy: String cr! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/6/2016 13:23'!
backspace

	| prior |
	self cursor x = 0 ifTrue: [^ self].
	prior := self currentLine contents.
	self currentLine contents: (prior copyFrom: 1 to: self cursor x - 1), (prior copyFrom: self cursor x + 1 to: prior size).! !

!SqshModel methodsFor: 'bindings' stamp: 'mt 2/14/2016 13:34'!
bindingOf: aString 

	(self bindings includesKey: aString) ifFalse: [
		aString first isUppercase
			ifTrue: [^nil]
			ifFalse: [bindings at: aString put: nil]].
		
	^ bindings associationAt: aString! !

!SqshModel methodsFor: 'bindings - accessing' stamp: 'mt 2/14/2016 13:32'!
bindings

	^ bindings! !

!SqshModel methodsFor: 'bindings - accessing' stamp: 'mt 2/14/2016 13:32'!
bindings: anObject

	bindings := anObject! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 11:19'!
clearEditFlag
	edits := false.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 14:17'!
clearLines

	self lines removeAll.
	self changed: #lines.! !

!SqshModel methodsFor: 'bindings - accessing' stamp: 'mt 2/14/2016 13:29'!
context

	^ context! !

!SqshModel methodsFor: 'bindings - accessing' stamp: 'mt 2/14/2016 13:29'!
context: anObject

	context := anObject! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:05'!
currentLine
	^ currentLine! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 7/8/2016 11:46'!
currentLine: aLine
	currentLine := aLine.
	self cursor: (self cursor x min: aLine contents size) @ (self lines indexOf: aLine ifAbsent: [0]).! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:15'!
cursor

	^ cursor! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:15'!
cursor: anObject

	cursor := anObject! !

!SqshModel methodsFor: 'testing' stamp: 'mt 7/8/2016 13:46'!
cursorAtEndOfLine
	^ self cursor x >= self currentLine contents size! !

!SqshModel methodsFor: 'testing' stamp: 'mt 7/8/2016 13:48'!
cursorAtLastLine
	^ self cursor y >= self lines size! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/6/2016 13:22'!
forwardDelete

	| prior |
	prior := self currentLine contents.
	self cursor x >= prior size ifTrue: [^ self].
	self currentLine contents: (prior copyFrom: 1 to: self cursor x), (prior copyFrom: self cursor x + 2 to: prior size).! !

!SqshModel methodsFor: 'bindings' stamp: 'mt 2/14/2016 13:34'!
hasBindingOf: aString 

	^bindings includesKey: aString.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 11:19'!
hasEdits
	^ edits! !

!SqshModel methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 12:34'!
initialize

	super initialize.
	
	self
		lines: OrderedCollection new;
		cursor: 0@1;
		bindings: OrderedDictionary new;
		clearEditFlag.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/6/2016 13:22'!
insert: string
	"Insert string at cursor position."
	
	| prior |
	prior := self currentLine contents.
	self currentLine contents: (prior copyFrom: 1 to: self cursor x), string, (prior copyFrom: self cursor x + 1 to: prior size).! !

!SqshModel methodsFor: 'adding' stamp: 'mt 7/8/2016 11:28'!
insertLine: stringOrText

	^ (self insertLines: {stringOrText}) first! !

!SqshModel methodsFor: 'adding' stamp: 'mt 7/8/2016 11:28'!
insertLines: someLines
	"... below the cursor position."
	
	| result |
	result := (someLines collect: [:line |
		SqshLine new
			visible: true;
			contents: line;
			prefix: '';
			addDependent: self;
			yourself]).
		
	result doWithIndex: [:line :index |
		self lines add: line afterIndex: self cursor y + index - 1].
		
	self changed: #lines.
	^ result! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 13:44'!
joinBackward
	"Join the current line at the cursor position. Removes a line."
	
	| prior current newX |
	(self cursor x = 0 and: [self cursor y > 1]) ifTrue: [
		prior := self lines at: (self lines indexOf: self currentLine) - 1.
		current := self currentLine.
	
		newX := prior contents size.
		prior contents: prior contents, current contents.
		self currentLine: prior.
	
		self lines remove: current.
	
		self cursor: newX @ self cursor y.
		self changed: #lines.
		^ true].

	^ false! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 13:44'!
joinForeward
	"Join the current line at the cursor position. Removes a line."
	
	| next current |
	(self cursor x = self currentLine contents size and: [self cursor y < self lines size]) ifTrue: [
		current := self currentLine.
		next := self lines at: (self lines indexOf: self currentLine) + 1.
	
		current contents: current contents, next contents.	
		self lines remove: next.
		self changed: #lines.
		^ true].	
	
	^ false! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/18/2016 19:41'!
lastResult

	^ (self bindingOf: '_result') value! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:54'!
lastResult: anObject

	(self bindingOf: '_result') value: anObject.! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:56'!
lines

	^ lines! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:06'!
lines: anObject

	lines := anObject! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 11:45'!
moveCursorDown

	self lines size > self cursor y ifTrue: [
		self currentLine: (self lines at: self cursor y + 1).
		self changed: #cursor].! !

!SqshModel methodsFor: 'editing' stamp: 'mt 5/7/2016 11:44'!
moveCursorEnd

	self cursor: (self lines at: self cursor y) contents size @ self cursor y.
	self changed: #cursor.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:08'!
moveCursorLeft

	self cursor: (self cursor x - 1 max: 0) @ self cursor y.
	self changed: #cursor.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 5/7/2016 11:45'!
moveCursorPos1

	self cursor: 0 @ self cursor y.
	self changed: #cursor.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 5/7/2016 12:19'!
moveCursorRight

	self cursor: (self cursor x + 1 min: self currentLine contents size) @ self cursor y.
	self changed: #cursor.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 11:45'!
moveCursorUp

	self cursor y > 1 ifTrue: [
		self currentLine: (self lines at: self cursor y - 1).
		self changed: #cursor].! !

!SqshModel methodsFor: 'bindings - accessing' stamp: 'mt 5/7/2016 12:26'!
receiver

	^ receiver! !

!SqshModel methodsFor: 'bindings - accessing' stamp: 'mt 5/7/2016 12:25'!
receiver: anObject

	receiver := anObject! !

!SqshModel methodsFor: 'adding' stamp: 'mt 7/8/2016 14:21'!
replaceLines: newLines

	newLines ifEmpty: [^ self replaceLines: #('')].
	self lines removeAll.
	self addLines: newLines.
	self currentLine: self lines first.
	^ self lines copy! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 11:18'!
setEditFlag
	edits := true.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 7/8/2016 11:28'!
split
	"Split the current line at the cursor position."
	
	| prior |
	prior := self currentLine contents.
	self currentLine contents: (prior copyFrom: 1 to: self cursor x).
	self insertLine: (prior copyFrom: self cursor x + 1 to: prior size).
	
	self
		moveCursorDown;
		moveCursorPos1.! !

!SqshModel methodsFor: 'updating' stamp: 'mt 7/8/2016 11:19'!
update: aspect

	"We give value holders to the outside so that controllers etc. can modify lines."
	aspect = #contents ifTrue: [
		self setEditFlag.
		self changed: #lines].! !

Object subclass: #SqshView
	instanceVariableNames: 'model defaultTextFont defaultTextColor defaultTextFillStyle scrollingOffset autoScroll fillStyle margin subviews superview bounds minimumExtent showCursor visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Core'!

!SqshView commentStamp: 'mt 3/6/2016 16:42' prior: 0!
I am a view that can display lines from a model. I can scroll.!

!SqshView methodsFor: 'display - support' stamp: 'mt 7/6/2016 14:56'!
alignmentFor: text

	(text attributesAt: 1) in: [:attrs |
		^ attrs detect: [:ea | ea class == TextAlignment]
			ifFound: [:ea | ea alignment = 1 ifTrue: [#right] ifFalse: [#left] ]
			ifNone: [#left] ].! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:20'!
autoScroll

	^ autoScroll! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/7/2016 15:12'!
autoScroll: boolean

	autoScroll := boolean.
	self invalidate.! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/6/2016 14:14'!
bounds
	^ bounds! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/8/2016 15:26'!
bounds: aRectangle
	bounds := aRectangle.
	
	self scrollingOffset: (self autoScroll
		ifTrue: [self offsetForAutoScroll]
		ifFalse: [self bounds topLeft + self margin topLeft]).
		
	self invalidate.! !

!SqshView methodsFor: 'display - support' stamp: 'mt 7/6/2016 14:56'!
colorFor: text

	(text attributesAt: 1) in: [:attrs |
		^ attrs detect: [:ea | ea class == TextColor]
			ifFound: [:ea | ea color]
			ifNone: [self defaultTextColor] ].! !

!SqshView methodsFor: 'defaults' stamp: 'mt 7/6/2016 13:07'!
defaultLineHeight

	^ self defaultTextFont height! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/6/2016 13:07'!
defaultTextColor

	^ defaultTextColor! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/7/2016 15:12'!
defaultTextColor: aColor

	defaultTextColor := aColor.
	self invalidate.! !

!SqshView methodsFor: 'defaults' stamp: 'mt 7/8/2016 10:48'!
defaultTextCursorColor

	^  (Color r: 0.149 g: 0.545 b: 0.823)! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 13:50'!
defaultTextFillStyle

	^ defaultTextFillStyle! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 13:50'!
defaultTextFillStyle: aFillStyle

	defaultTextFillStyle := aFillStyle.
	self invalidate.! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/6/2016 13:07'!
defaultTextFont

	^ defaultTextFont! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 10:09'!
defaultTextFont: aFont

	defaultTextFont := aFont.
	self minimumExtent: (aFont widthOf: $x) * 5 @ 0.
	
	self invalidate.! !

!SqshView methodsFor: 'display' stamp: 'mt 7/8/2016 13:30'!
displayContentsOn: canvas
	
	self fillStyle ifNotNil: [:fs | canvas fillRectangle: self bounds fillStyle: fs].
	
	"Avoid text layouting issues if we are too narrow."
	self width < self minimumExtent x ifTrue: [^ self].
	
	canvas
		clipBy: (self bounds insetBy: self margin)
		during: [:clippedCanvas | self displayLinesOn: clippedCanvas].! !

!SqshView methodsFor: 'display' stamp: 'mt 7/8/2016 13:58'!
displayCursorAt: tmp on: canvas
	
	self showCursor ifFalse: [^ self].
	canvas
		line: tmp + (0@1)  to: tmp + (0@ -1) + (0 @ self defaultLineHeight) color: self defaultTextCursorColor;
		line: tmp + (1@1) to: tmp + (1@ -1) + (0 @ self defaultLineHeight) color: (self defaultTextCursorColor alpha: 0.5).! !

!SqshView methodsFor: 'display' stamp: 'mt 7/8/2016 13:54'!
displayLine: text at: pt on: canvas
	"Adds support for line breaks. Works best with monospaced fonts (for performance reasons). Returns resulting offset and intervals."

	| charactersPerLine actualWidth position o i w font color leftOrRight |
	color := self colorFor: text.
	font := self fontFor: text.
	leftOrRight := self alignmentFor: text.
	
	actualWidth := self width - self margin left - self margin right.
	
	text ifEmpty: [
		"Background for lines."
		self defaultTextFillStyle ifNotNil: [:fs |
			canvas fillRectangle: (pt extent: actualWidth @ font height) fillStyle: fs].
				
		^ {pt + (0@ font height). {1 to: text size}}].
	
	charactersPerLine := actualWidth // (font widthOf: $x).
	position := 1.
	
	"Display text line with line breaks."
	i := OrderedCollection new.
	o := pt.	
	[position <= text size] whileTrue: [
		i add: (position to: (position + charactersPerLine - 1 min: text size)).
		
		w := leftOrRight == #right ifFalse: [0] ifTrue: [actualWidth - (font widthOfString: text string from: i last first to: i last last)].
		
		"Background for lines."
		self defaultTextFillStyle ifNotNil: [:fs |
			canvas fillRectangle: ((o x + w) @ o y extent: actualWidth @ font height) fillStyle: fs].
		
		canvas drawString: text string from: i last first to: i last last at: (o x + w) @ o y font: font color: color.
		o := o + (0@ font height).
		position := position + charactersPerLine].
	
	^ {o. i}! !

!SqshView methodsFor: 'display' stamp: 'mt 7/8/2016 14:57'!
displayLinesOn: canvas
	
	| offset cursor prev |
	offset := self autoScroll "autoscroll means show prompt or cursor if visible?"
		ifTrue: [self offsetForAutoScroll]
		ifFalse: [self scrollingOffset].
	
	self model lines doWithIndex: [:line :index | line visible ifTrue: [
		
			
		line prefix, line contents
			ifEmpty: [ | newOffset |
				newOffset := (self displayLine: '' asText at: offset on: canvas) first.
				self model cursor = (0@index)
					ifTrue: [self displayCursorAt: offset on: canvas].
				offset := newOffset]
			ifNotEmpty: [:text | 
				"Update text cursor according to prefix."
				index = self model cursor y
					ifTrue: [cursor := (self model cursor x + line prefix size) @ self model cursor y].
			
				text string lineIndicesDo: [:start :end :endCr |
					| part |
					prev := offset.
					part := text copyFrom: start to: end.
					offset := (self displayLine: part at: offset on: canvas).

					"Draw text cursor considering soft line breaks."
					index = self model cursor y ifTrue: [
						| intervals tmp |		
						intervals := offset second.
						tmp := prev.
						((cursor x max: 1) between: start and: endCr)
							ifTrue: [
								"Found the line with the text cursor in it!! Show it."
								intervals
									detect: [:i |  (i includes: cursor x-start+1) ifTrue: [true] ifFalse: [
										tmp := tmp + (0@ self defaultLineHeight).
										false]]
									ifFound: [:i |
										tmp := self margin left + ((self fontFor: part) widthOfString: part from: i first to: cursor x-start+1) @ tmp y]
									ifNone: [].
								cursor x = 0 ifTrue: [tmp := self margin left @ prev y].				
								self displayCursorAt: tmp on: canvas.
								]].
			
					offset := offset first. ]]]].! !

!SqshView methodsFor: 'display' stamp: 'mt 7/8/2016 14:24'!
displayOn: canvas
	
	self visible ifFalse: [^ self].
	(canvas isVisible: self bounds) ifFalse: [^ self].
	
	self
		displayContentsOn: canvas;
		displaySubviewsOn: canvas.! !

!SqshView methodsFor: 'display' stamp: 'mt 7/6/2016 14:07'!
displaySubviewsOn: canvas
	
	self subviews do: [:view | view displayOn: canvas].! !

!SqshView methodsFor: 'display - support' stamp: 'mt 7/8/2016 14:07'!
estimatedLinesHeight
	"Calculates the height for line breaks. Use own font or fallback if not specified."

	^ self model lines
		inject: 0
		into: [:sum :line | | cpl fontToUse |
			fontToUse := self fontFor: line contents.
			cpl := self width - self margin left - self margin right // (fontToUse widthOf: $x).
			(line contents string
				ifEmpty: [{1}]
				ifNotEmpty: [:string | string lines collect: [:l | (l size / cpl) ceiling max: 1 "empty lines"]]) sum
					* fontToUse height
			+ sum]
	

! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/6/2016 14:15'!
extent

	^ self bounds extent! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/6/2016 13:00'!
fillStyle
	^ fillStyle! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/7/2016 15:12'!
fillStyle: aFillStyle
	fillStyle := aFillStyle.
	self invalidate.! !

!SqshView methodsFor: 'display - support' stamp: 'mt 7/6/2016 14:56'!
fontFor: text

	(text attributesAt: 1) in: [:attrs |
		^ attrs detect: [:ea | ea class == TextFontReference]
			ifFound: [:ea | ea font]
			ifNone: [self defaultTextFont] ].! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/6/2016 14:14'!
height

	^ self bounds height! !

!SqshView methodsFor: 'initialize-release' stamp: 'mt 7/8/2016 15:24'!
initialize
	
	super initialize.
	self
		showCursor: true;
		margin: (0@0 corner: 0@0);
		autoScroll: true;
		defaultTextFont: Preferences standardCodeFont;
		defaultTextColor: (Color gray: 0.1);
		fillStyle: (SolidFillStyle color: Color white);
		visible: true;
		subviews: #().! !

!SqshView methodsFor: 'display' stamp: 'mt 7/8/2016 09:45'!
invalidate
	"Assume that embedded views are always within the bounds of their superviews. Hence, let only the outermost superview communicate its bounds to invalidate/redraw."
	
	self superview
		ifNotNil: [:view | view invalidate]
		ifNil: [self triggerEvent: #invalidate with: self bounds].! !

!SqshView methodsFor: 'accessing' stamp: 'mt 3/6/2016 17:14'!
margin

	^ margin! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/7/2016 15:12'!
margin: rect
	"Margin can be used to limit the space for contents while still displaying the fillstyle."
	
	margin := rect.
	self invalidate.! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/8/2016 10:08'!
minimumExtent
	^ minimumExtent! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/8/2016 10:09'!
minimumExtent: aRectangle
	minimumExtent := aRectangle.
	self invalidate.! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
model

	^ model! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/7/2016 15:12'!
model: anObject

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].

	model := anObject.
	self invalidate.! !

!SqshView methodsFor: 'scrolling' stamp: 'mt 7/8/2016 13:06'!
offsetForAutoScroll

	^ self bounds topLeft + (self margin left @ 0) + (0 @ ((self height - self margin bottom - self estimatedLinesHeight) min: self margin top))
! !

!SqshView methodsFor: 'scrolling' stamp: 'mt 2/14/2016 16:40'!
scrollDown: lines

	self autoScroll ifTrue: [^ self].

	self offsetForAutoScroll in: [:o |
		self scrollingOffset: self scrollingOffset - (0@ (self defaultLineHeight * lines)).		
		self scrollingOffset <= o ifTrue: [self autoScroll: true]].
! !

!SqshView methodsFor: 'scrolling' stamp: 'mt 7/6/2016 15:38'!
scrollUp: lines

	self autoScroll ifTrue: [self scrollingOffset: self offsetForAutoScroll].

	self autoScroll: false.
	self scrollingOffset: ((self scrollingOffset + (0@ (self defaultLineHeight * lines))) min: self bounds topLeft + (0@self margin top)).! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:16'!
scrollingOffset
	^ scrollingOffset! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/7/2016 15:12'!
scrollingOffset: point

	scrollingOffset := point.
	self invalidate.! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 13:57'!
showCursor
	^ showCursor! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 13:57'!
showCursor: aBoolean
	showCursor := aBoolean.
	self invalidate.! !

!SqshView methodsFor: 'composition' stamp: 'mt 7/6/2016 14:07'!
subviews
	^ subviews! !

!SqshView methodsFor: 'composition' stamp: 'mt 7/7/2016 15:12'!
subviews: someViews
	subviews := someViews.
	someViews do: [:view | view superview: self].
	self invalidate.! !

!SqshView methodsFor: 'composition' stamp: 'mt 7/6/2016 14:50'!
superview

	^ superview! !

!SqshView methodsFor: 'composition' stamp: 'mt 7/6/2016 14:50'!
superview: anObject

	superview := anObject! !

!SqshView methodsFor: 'display' stamp: 'mt 7/7/2016 15:26'!
update: aspect

	(#(cursor lines) includes: aspect)
		ifTrue: [self invalidate].! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:23'!
visible
	^ visible! !

!SqshView methodsFor: 'accessing' stamp: 'mt 7/8/2016 14:23'!
visible: aBoolean
	visible := aBoolean.
	self invalidate.! !

!SqshView methodsFor: 'geometry' stamp: 'mt 7/6/2016 14:14'!
width

	^ self bounds width! !

Object subclass: #SqueakShell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Core'!

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 10:59'!
logo

	^ '
       ###                                                          ####        
     ########                                                    ####  ###      
    ##      ###                                                ###      ###     
   ###       ###                                              ###        ##     
   ##          ###                                           ##          ##     
   ##           ###                                        ###            ##    
   ##            ###                                      ###             ##    
   ##             ###                                    ###              ##    
   ##              ###                                  ###               ##    
   ##               ###                                ###               ##     
   ##                ###                              ###                ##     
   ##                 ###                            ###                 ##     
   ##                  ###                           ##                  ##     
    ##                  ##                           #                   ##     
    ##                                                                  ##      
    ##                                                                  ##      
     ##                                                                 ##      
     ##                                                                ##       
     ###                  ######               ######                  ##       
      ##                ##########            ########                ###       
      ##               ###########           ##########               ##        
       ##              ####### ####         ####### ###              ###        
       ##              ######   ###         ######   ###             ##         
                       ####### ###          ####### ####                        
                        ##########          ###########                         
                         ########            #########                          
                           ####               #######                           
                                                                                
                                                                                
     #####################           ######                                     
##########            #########   ###########     ###########################   
##                           ##  #############  #######                 ########
            ##################   ##############                                 
      #############   ########   ##############  #####################          
  ######                          ############                     #######      
####             ############      #########                             #####  
#           ##################                   ###############            ####
          ####                                   #            ######           #
        ###                                                       ####          
      ###                                                           ####        
                                                                      ##        '! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 10:42'!
open
	"SqueakShell open."
	
	| world |
	world := SqshControlManager new.
	world activeController captureDisplay.
	
	[world exitValue] whileNil: [world doOneCycle. Processor yield].
	Project current restoreDisplay.
	^ world exitValue! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:16'!
openAsProject

	SqshProject new enter.! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 10:03'!
openConcurrent
	"SqueakShell open."
	
	| world |
	world := SqshControlManager new.
	world activeController captureDisplay.
	
	[
		[world exitValue] whileNil: [world doOneCycle. Processor yield].
		Project current uiProcess resume.
		Project current restoreDisplay.
	] forkAt: Processor userSchedulingPriority.

	Project current uiProcess suspend.! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 17:18'!
openOnStdio
	"SqueakShell openOnStdio."
	
	| world |
	world := SqshStdioManager new.
	
	[world exitValue] whileNil: [world doOneCycle. Processor yield].
	Project current restoreDisplay.
	^ world exitValue! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 7/6/2016 15:24'!
showStack: methodContext
	"Debugging."
	
	(methodContext stackOfSize: 20)
		collect: [:ctxt | ctxt printString]
		thenDo: [:string | Project current world activeController showWarning: string].! !

!Debugger class methodsFor: '*SqueakShell-Support' stamp: 'mt 2/15/2016 15:44'!
sqshContext: aContext
	"Answer an instance of me for debugging the active process starting with the given context."

	^ self new
		process: Processor activeProcess
		controller: nil
		context: aContext! !

!Debugger class methodsFor: '*SqueakShell-Support' stamp: 'mt 7/6/2016 15:23'!
sqshOpenContext: aContext label: aString contents: contentsStringOrNil

	| process errorWasInUIProcess |
	process := Processor activeProcess.
	errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: process.

	ErrorRecursion not & Preferences logDebuggerStackToFile ifTrue:
		[Smalltalk logSqueakError: aString inContext: aContext].
	ErrorRecursion ifTrue:[
		ErrorRecursion := false.
		self primitiveError: aString].
	ErrorRecursion := true.

	Project current addDeferredUIMessage: [
			UIManager default
				inform: aString;
				inform: contentsStringOrNil.
			SqueakShell showStack: aContext.
			process terminate. "No support for debugging yet."
	].

	ErrorRecursion := false.

	process suspend.! !

!Debugger class methodsFor: '*SqueakShell-Support' stamp: 'mt 7/6/2016 15:23'!
sqshOpenInterrupt: aString onProcess: interruptedProcess
	"Open a notifier in response to an interrupt. An interrupt occurs when the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other systems) or when the low-space watcher detects that memory is low."

	| errorWasInUIProcess |
	self assert: interruptedProcess isSuspended.
	
	errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: interruptedProcess.

	Preferences logDebuggerStackToFile ifTrue:
		[(aString includesSubstring: 'Space') & (aString includesSubstring: 'low')
			ifTrue: [Smalltalk logError: aString inContext: interruptedProcess suspendedContext to: 'LowSpaceDebug.log']
			ifFalse: [Smalltalk logSqueakError: aString inContext: interruptedProcess suspendedContext]].
		
	Project current addDeferredUIMessage: [
			UIManager default inform: aString.
			SqueakShell showStack: interruptedProcess suspendedContext.
			interruptedProcess terminate. "No support for debugging yet."
	].! !

!Debugger class methodsFor: '*SqueakShell-Support' stamp: 'mt 7/6/2016 15:24'!
sqshOpenOn: process context: context label: title contents: contentsStringOrNil fullView: bool

	| errorWasInUIProcess |
	errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: process.
	
	[
		Preferences logDebuggerStackToFile
			ifTrue: [Smalltalk logSqueakError: title inContext: context].			
	] on: Error do: [:ex | ex return: nil].

	Project current addDeferredUIMessage: [
		[
			UIManager default inform: title.
			contentsStringOrNil ifNotNil: [:s | UIManager default inform: s].
			
			SqueakShell showStack: context.
			process terminate. "No support for debugging yet."
			
		] on: Error do: [:ex |
					self primitiveError: 
						'Original error: ', 
						title asString, '.
		Debugger error: ', 
					([ex description] on: Error do: ['a ', ex class printString]), ':'
				].
	].

	process suspend.! !

!PluggableFileList methodsFor: '*SqueakShell-Support' stamp: 'mt 4/14/2016 16:18'!
sqshOpenLabel: aString in: aWorld

	UIManager default
		inform: self directory printString;
		inform: (self fileList joinSeparatedBy: String cr).! !

Model subclass: #SqshLine
	instanceVariableNames: 'contents font color alignment prefix visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Core'!

!SqshLine class methodsFor: 'instance creation' stamp: 'mt 2/18/2016 09:55'!
contents: string

	^ self new contents: string! !

!SqshLine class methodsFor: 'instance creation' stamp: 'mt 2/18/2016 09:56'!
contents: string font: font

	^ self new contents: string; font: font! !

!SqshLine class methodsFor: 'instance creation' stamp: 'mt 2/18/2016 10:10'!
contents: string font: font color: color

	^ self new contents: string; font: font; color: color! !

!SqshLine methodsFor: 'accessing' stamp: 'mt 2/18/2016 09:55'!
contents

	^ contents! !

!SqshLine methodsFor: 'accessing' stamp: 'mt 7/6/2016 15:35'!
contents: anObject

	contents = anObject ifTrue: [^ self].
	contents := anObject asText.
	self contentsChanged.! !

!SqshLine methodsFor: 'accessing' stamp: 'mt 5/7/2016 11:04'!
prefix

	^ prefix! !

!SqshLine methodsFor: 'accessing' stamp: 'mt 7/6/2016 15:35'!
prefix: anObject

	prefix = anObject ifTrue: [^ self].
	prefix := anObject asText.
	self contentsChanged.! !

!SqshLine methodsFor: 'printing' stamp: 'mt 7/6/2016 16:06'!
printOn: stream

	self contents asString printOn: stream.! !

!SqshLine methodsFor: 'attributes' stamp: 'mt 7/6/2016 15:00'!
setBold
	self contents addAttribute: TextEmphasis bold.! !

!SqshLine methodsFor: 'attributes' stamp: 'mt 7/6/2016 15:00'!
setColor: aColor
	self contents addAttribute: (TextColor color: aColor).! !

!SqshLine methodsFor: 'attributes' stamp: 'mt 7/6/2016 15:00'!
setItalic
	self contents addAttribute: TextEmphasis italic.! !

!SqshLine methodsFor: 'attributes' stamp: 'mt 7/6/2016 15:01'!
setLeftFlush
	self contents addAttribute: TextAlignment leftFlush.! !

!SqshLine methodsFor: 'attributes' stamp: 'mt 7/6/2016 15:33'!
setMonospaced
	self contents addAttribute: (TextFontReference toFont: (StrikeFont familyName: 'BitstreamVeraSansMono' pointSize: 12)).! !

!SqshLine methodsFor: 'attributes' stamp: 'mt 7/6/2016 15:01'!
setRightFlush
	self contents addAttribute: TextAlignment rightFlush.! !

!SqshLine methodsFor: 'accessing' stamp: 'mt 7/6/2016 15:50'!
visible
	^ visible! !

!SqshLine methodsFor: 'accessing' stamp: 'mt 7/6/2016 15:50'!
visible: aBoolean
	visible := aBoolean.! !

FormCanvas subclass: #SqshStdioCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Stdio'!

!SqshStdioCanvas methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 17:52'!
drawString: s from: a to: b at: pt font: font color: color

	FileStream stdout nextPut: Character cr.
	FileStream stdout nextPutAll: s.
	
"	FileStream stdout nextPut: #[27 69]; ascii."! !

!SqshStdioCanvas methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2016 17:09'!
fillRectangle: rect fillStyle: fs
	"Ignore."! !

!Collection methodsFor: '*SqueakShell-Support' stamp: 'mt 2/16/2016 10:40'!
printLines

	^ String streamContents: [:strm |
		self do: [:ea | strm nextPutAll: ea printString; cr]]! !

!Collection methodsFor: '*SqueakShell-Support' stamp: 'mt 2/18/2016 19:45'!
printNumberedLines

	| i len |
	i := 1.
	len := self size asString size.

	^ String streamContents: [:strm |
		self do: [:ea | strm
			nextPutAll: (i asString padded: #left to: len with: Character space);
			nextPut: Character space;
			nextPutAll: ea printString; cr.
			i := i + 1]]! !

StandardToolSet subclass: #SqshToolSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Support'!

!SqshToolSet class methodsFor: 'inspecting' stamp: 'mt 4/18/2016 16:39'!
basicInspect: anObject
	"Bypass tool builder framework."
	
	self inspect: anObject.! !

!SqshToolSet class methodsFor: 'browsing' stamp: 'mt 4/18/2016 16:39'!
browseClass: class
	"Bypass tool builder framework."

	UIManager default inform: class selectors.! !

!SqshToolSet class methodsFor: 'browsing' stamp: 'mt 4/18/2016 16:39'!
browseMessageNames: aString
	"Bypass tool builder framework."

	UIManager default inform: (MessageNames new
		computeSelectorListFrom: aString)! !

!SqshToolSet class methodsFor: 'as yet unclassified' stamp: 'mt 2/18/2016 19:17'!
controller

	^ Project current world activeController! !

!SqshToolSet class methodsFor: 'inspecting' stamp: 'mt 4/18/2016 16:39'!
explore: anObject
	"Bypass tool builder framework."

	self inspect: anObject.! !

!SqshToolSet class methodsFor: 'inspecting' stamp: 'mt 7/6/2016 15:21'!
inspect: anObject
	"Bypass tool builder framework."

	(self controller showMessage: (anObject longPrintStringLimitedTo: 20000))
		setMonospaced.! !

!MailComposition methodsFor: '*SqueakShell-Support' stamp: 'mt 4/14/2016 16:22'!
sqshOpen

	UIManager default inform: 'Sorry, there is no support for mail send in the Squeak Shell.'.! !

UIManager subclass: #SqshUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Support'!

!SqshUIManager class methodsFor: 'class initialization' stamp: 'mt 4/13/2016 16:45'!
isActiveManager

	^ Project current isSqueakShell! !

!SqshUIManager methodsFor: 'ui requests' stamp: 'mt 2/15/2016 18:22'!
displayProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock
	
	^ Project current world activeController
		showProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock.! !

!SqshUIManager methodsFor: 'ui requests' stamp: 'mt 2/15/2016 18:22'!
inform: msg

	Project current world activeController showMessage: msg.! !

!SqshUIManager methodsFor: 'ui requests' stamp: 'mt 2/15/2016 18:22'!
informUserDuring: block

	Project current world activeController showMessageDuring: block.! !

!SyntaxError class methodsFor: '*SqueakShell-Support' stamp: 'mt 5/9/2016 12:51'!
sqshOpen: aSyntaxError

	| process |
	process := Processor activeProcess.
	
	Project current spawnNewProcessIfThisIsUI: process.

	Project current addDeferredUIMessage: [
		self inform: aSyntaxError contents.
		process terminate.
	].
			
	process suspend.! !

!Project methodsFor: '*SqueakShell-testing' stamp: 'mt 4/12/2016 18:58'!
isSqueakShell

	^ false! !

Project subclass: #SqshProject
	instanceVariableNames: 'uiProcess previousToolSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell-Support'!

!SqshProject methodsFor: 'scheduling' stamp: 'mt 2/15/2016 18:13'!
addDeferredUIMessage: valuableObject 

	self world activeController deferredMessages nextPut: valuableObject.! !

!SqshProject methodsFor: 'debugging' stamp: 'mt 4/18/2016 16:41'!
debugMethod: aCompiledMethod forReceiver: anObject inContext: aContextOrNil

	self uiManager
		inform: 'No debugging support. Just evaluating the expression...';
		inform: (super debugMethod: aCompiledMethod forReceiver: anObject inContext: aContextOrNil) printString.
		
	"How to store the result in the current controller?"! !

!SqshProject methodsFor: 'enter' stamp: 'mt 4/17/2016 19:03'!
finalEnterActions: leavingProject
	
	super finalEnterActions: leavingProject.
	transcript addDependent: self world activeController.
	
	"Override the standard tool set."
	previousToolSet := ToolSet default.
	ToolSet default: SqshToolSet.! !

!SqshProject methodsFor: 'enter' stamp: 'mt 4/17/2016 18:24'!
finalExitActions: enteringProject

	super finalExitActions: enteringProject.
	transcript removeDependent: self world activeController.
	ToolSet default: previousToolSet.! !

!SqshProject methodsFor: 'initialization' stamp: 'mt 3/1/2016 17:19'!
initialize

	super initialize.

	world := SqshControlManager new.! !

!SqshProject methodsFor: 'debugging' stamp: 'mt 2/15/2016 16:03'!
interruptName: labelString
	"Create a Notifier on the active scheduling process with the given label."

	^ self interruptName: labelString preemptedProcess: nil! !

!SqshProject methodsFor: 'debugging' stamp: 'mt 2/15/2016 16:13'!
interruptName: labelString preemptedProcess: theInterruptedProcess
	"Create a Notifier on the active scheduling process with the given label."

	| preemptedProcess projectProcess |
	projectProcess := self uiProcess.
	
	preemptedProcess := theInterruptedProcess ifNil: [Processor preemptedProcess].
	"Only debug preempted process if its priority is >= projectProcess' priority"
	
	preemptedProcess priority < projectProcess priority 
		ifTrue:[preemptedProcess := projectProcess].
	
	preemptedProcess suspend.
	ToolSet interrupt: preemptedProcess label: labelString.! !

!SqshProject methodsFor: 'displaying' stamp: 'mt 4/13/2016 11:10'!
invalidate

	self world reset.! !

!SqshProject methodsFor: 'testing' stamp: 'mt 4/12/2016 18:58'!
isSqueakShell

	^ true! !

!SqshProject methodsFor: 'utilities' stamp: 'mt 4/17/2016 19:16'!
jumpToProject
! !

!SqshProject methodsFor: 'displaying' stamp: 'mt 7/7/2016 14:38'!
restore

	self world display.! !

!SqshProject methodsFor: 'enter' stamp: 'mt 4/17/2016 19:03'!
scheduleProcessForEnter

	self invalidate.
	self spawnNewProcess.! !

!SqshProject methodsFor: 'dispatching' stamp: 'mt 2/15/2016 15:17'!
selectorPrefixForDispatch

	^ 'sqsh'! !

!SqshProject methodsFor: 'active process' stamp: 'mt 2/15/2016 18:05'!
spawnNewProcess

	uiProcess := [
		[
			DisplayScreen checkForNewScreenSize.			
			self world doOneCycle.			
			Processor yield.
		] repeat.
	] newProcess.

	uiProcess
		priority: Processor userSchedulingPriority;
		resume.! !

!SqshProject methodsFor: 'active process' stamp: 'mt 2/12/2016 16:28'!
spawnNewProcessIfThisIsUI: suspendedProcess
	"Initialize a UI process if needed. Answer true if suspendedProcess was interrupted
	from a UI process."

	self uiProcess == suspendedProcess ifTrue: [
		self spawnNewProcess.
		^true
	].
	^false		"no new process was created"
! !

!SqshProject methodsFor: 'enter' stamp: 'mt 4/17/2016 22:00'!
suspendProcessForDebug

	| p |
	self assert: Processor activeProcess == uiProcess.
	
	p := uiProcess.
	uiProcess := nil.
	p suspend.! !

!SqshProject methodsFor: 'enter' stamp: 'mt 4/12/2016 18:23'!
terminateProcessForLeave

	Processor terminateActive.! !

!SqshProject methodsFor: 'active process' stamp: 'mt 2/12/2016 13:01'!
uiProcess

	^ uiProcess! !
